/* Activate step incorrectly adds old Linked Page URLs to DocumentAlias, leading to potential 
	conflicts on subsequent activations. This script patches the Activation procedure and 
	removes all linked pages from Document Alias */
	
ALTER PROCEDURE internal_Activate @eventID UNIQUEIDENTIFIER, @notes NVARCHAR(MAX), @docsToActivate XML
AS
BEGIN
SET NOCOUNT ON;
    IF @docsToActivate IS NULL
        RETURN; 
		
	DECLARE @docsToOp XML (TitanDocsXml);
	SET @docsToOp = @docsToActivate

    EXEC dbo.internal_WriteWorkstationAuditEvent @eventID, 'InternalFunction', 'internal_Activate', 'Incoming Doc Set', @docsToOp;

    DECLARE @actionTime DATETIME;
    SET @actionTime = GetDate();

    DECLARE @docs TABLE 
    (
        tDocID    INT NOT NULL, 
        tVersion  INT NOT NULL,
        PRIMARY KEY (tDocID, tVersion)
    );
    INSERT INTO @docs (tDocID, tVersion)
         SELECT OneRow.value('.', 'INT'), OneRow.value('@Version', 'INT')
           FROM @docsToOp.nodes('/Docs/DocID') AS DocIDs(OneRow);

	/* v5.8 - Active Version should never be a prior version, though it could be the first version
	in which case Document.ActiveVersion and the incoming version are both = 1... so you can't do <= */
	DELETE FROM @docs
	  FROM Document
	 WHERE tDocID   = Document.DocID
	   AND tVersion < Document.ActiveVersion
	   
    DELETE FROM @docs
      FROM NavigationVersion
     WHERE tDocID   = NavigationVersion.DocID
       AND tVersion = NavigationVersion.Version
       AND NOT (GETDATE() BETWEEN NavigationVersion.StartDate AND NavigationVersion.EndDate);

    DELETE FROM @docs
      FROM Document
           JOIN NavigationVersion
             ON Document.DocID         = NavigationVersion.DocID
            AND Document.ActiveVersion = NavigationVersion.Version 
     WHERE tDocID   = Document.DocID
       AND tVersion = Document.ActiveVersion
       AND NavigationVersion.CurrentWorkflowStatus != 3;

    DELETE FROM @docs
      FROM Workflow
     WHERE tDocID = Workflow.DocID
       AND tVersion = Workflow.Version
       AND Workflow.IsActive = 1; 

	/* v5.8 - Active Version determined primarily by max version number, no longer by most future start date */
    DELETE FROM @docs
      FROM (SELECT DocID, Version
              FROM (SELECT RANK() OVER (PARTITION BY NavigationVersion.DocID ORDER BY NavigationVersion.Version DESC) AS Rank, NavigationVersion.DocID, NavigationVersion.Version
                      FROM NavigationVersion  
                           JOIN @docs docs
                             ON docs.tDocID = NavigationVersion.DocID
                     WHERE @actionTime BETWEEN NavigationVersion.StartDate AND NavigationVersion.EndDate
                       AND NavigationVersion.CurrentWorkflowStatus=3) x
              WHERE Rank = 1) x
     WHERE tDocID = DocID
       AND tVersion != Version;

    IF NOT EXISTS (SELECT 1 FROM @docs)
        RETURN;

    /* Verify that the LinkParameter and Aliases are all unique.  
		- Check LinkParameter against other Docs LinkParameters
		- Check Aliases against other Docs LinkParameters
		- Check LinkParameter against other Docs Aliases
		- Check Aliases against other Docs Aliases
		- Check LinkParameter against own Aliases
    */
    -- collect all the links that we need to check
	DECLARE @links TABLE (LinkParameter NVARCHAR(440), DocID INT, Version INT, linkType VARCHAR(1), PRIMARY KEY (LinkParameter, DocID, Version, linkType));

	WITH ActiveDocsUrls AS (
		SELECT NavigationVersion.DocID, Navigationversion.Version, NavigationVersion.LinkParameter
		  FROM @docs docs
		  JOIN Document
			ON Document.DocID = docs.tDocID
		  JOIN NavigationVersion
			ON NavigationVersion.DocID = docs.tDocID
		   AND NavigationVersion.Version = docs.tVersion
		  JOIN DocumentVersion
			ON DocumentVersion.DocID = docs.tDocID
		   AND DocumentVersion.Version = docs.tVersion
		  JOIN DocumentType 
			ON DocumentType.DocumentTypeID = DocumentVersion.DocumentTypeID
		   AND DocumentType.SiteID = Document.SiteID
		 WHERE SupportsUrlRewriting = 1
		   AND Document.SiteID != -1
	)
	INSERT INTO @links (DocID, Version, LinkParameter, linkType)
		SELECT ActiveDocsUrls.DocID, ActiveDocsUrls.Version, ActiveDocsUrls.LinkParameter, 'L'
		  FROM ActiveDocsUrls
		 UNION -- Not ALL, keep unique urls for the DocID/Version being activated.
		SELECT ActiveDocsUrls.DocID, ActiveDocsUrls.Version, DocumentAlias.LinkParameter, 'A'
		  FROM DocumentAlias
		  JOIN ActiveDocsUrls
			ON ActiveDocsUrls.DocID = DocumentAlias.DocID;

	DECLARE @dupLinks 
	  TABLE (aDocID INT, aVersion INT, nDocID INT, nVersion INT, dupType VARCHAR(4));
	  
	WITH DupLinks AS (
		SELECT NavigationVersion.DocID, NavigationVersion.Version, NavigationVersion.LinkParameter
		  FROM NavigationVersion
		  JOIN Document
			ON Document.DocID         = NavigationVersion.DocID
		   AND Document.ActiveVersion = NavigationVersion.Version
		  JOIN DocumentVersion
			ON DocumentVersion.DocID   = Document.DocID
		   AND DocumentVersion.Version = Document.ActiveVersion
		  JOIN DocumentType
			ON DocumentType.DocumentTypeID = DocumentVersion.DocumentTypeID
		   AND DocumentType.SiteID         = Document.SiteID
		 WHERE DocumentType.SupportsUrlRewriting = 1
		   AND (DocumentType.DocumentTypeID = 1 OR DocumentType.DocumentTypeID > 8)
		   AND Document.SiteID != -1
	)
	INSERT INTO @dupLinks (aDocID, aVersion, nDocID, nVersion, dupType)
		SELECT DupLinks.DocID, DupLinks.Version, docs.DocID, docs.Version, 'LOL'
		  FROM DupLinks -- LP matches other doc's LP
		  JOIN @links docs
		    ON docs.linkType = 'L'
		   AND docs.LinkParameter = DupLinks.LinkParameter
		   AND docs.DocID        != DupLinks.DocID 
		 UNION -- Not ALL, report only unique duplicates
		SELECT docs.DocID, docs.Version, docs.DocID, docs.Version, 'LSA'
		  FROM @links docs -- LP matches self Alias
		  JOIN DocumentAlias
		    ON docs.linkType = 'L'
		   AND docs.DocID = DocumentAlias.DocID
		   AND docs.LinkParameter = DocumentAlias.LinkParameter
		 UNION
		SELECT DupLinks.DocID, DupLinks.Version, docs.DocID, docs.Version, 'LOA'
		  FROM DupLinks -- LP matches other doc's Aliases
		  JOIN DocumentAlias
		    ON DocumentAlias.DocID = DupLinks.DocID
		  JOIN @links docs
			ON docs.linkType = 'L'
		   AND docs.LinkParameter = DocumentAlias.LinkParameter
		   AND docs.DocID != DupLinks.DocID
		 UNION 
		SELECT DupLinks.DocID, DupLinks.Version, docs.DocID, docs.Version, 'AOL'
		  FROM DupLinks -- Alias matches other doc's LP
		  JOIN @links docs
		    ON docs.linkType = 'A'
		   AND docs.LinkParameter = DupLinks.LinkParameter
		   AND docs.DocID        != DupLinks.DocID 
		 UNION		
		SELECT DupLinks.DocID, DupLinks.Version, docs.DocID, docs.Version, 'AOA'
		  FROM DupLinks -- Alias matches other (active) doc's Aliases
		  JOIN DocumentAlias
		    ON DocumentAlias.DocID = DupLinks.DocID
		  JOIN @links docs
			ON docs.linkType = 'A'
		   AND docs.LinkParameter = DocumentAlias.LinkParameter
		   AND docs.DocID != DupLinks.DocID
		 UNION 
		SELECT Document.DocID, Document.ActiveVersion, docs.DocID, docs.Version, 'ARA'
		  FROM @links docs -- Alias matches other (recycled) doc's Alias
		  JOIN Document 
		    ON Document.DocID = docs.DocID
		   AND Document.SiteID = -1
		  JOIN DocumentAlias
		    ON DocumentAlias.DocID = Document.DocID
		   AND docs.linkType = 'A'
		   AND docs.LinkParameter = DocumentAlias.LinkParameter
		   
		  
	IF EXISTS (SELECT 1 FROM @dupLinks)
	BEGIN
		DECLARE @dups NVARCHAR(MAX);
		SELECT @dups =  COALESCE(@dups, '') + 
						CASE dupType
							WHEN 'LOL' THEN 'New Document Page URL (ID:' +  CAST(nDocID AS NVARCHAR(10)) + ',VER:' + CAST(nVersion AS NVARCHAR(10)) + ') conflicts with Active Document Page URL (ID:' +  CAST(aDocID AS NVARCHAR(10)) + ',VER:' + CAST(aVersion AS NVARCHAR(10)) + ')<br/>'
							WHEN 'LSA' THEN 'New Document Page URL (ID:' +  CAST(nDocID AS NVARCHAR(10)) + ',VER:' + CAST(nVersion AS NVARCHAR(10)) + ') conflicts with one if its own Alias URLs<br/>'
							WHEN 'LOA' THEN 'New Document Page URL (ID:' +  CAST(nDocID AS NVARCHAR(10)) + ',VER:' + CAST(nVersion AS NVARCHAR(10)) + ') conflicts with Active Document Alias URL (ID:' +  CAST(aDocID AS NVARCHAR(10)) + ',VER:' + CAST(aVersion AS NVARCHAR(10)) + ')<br/>'
							WHEN 'AOL' THEN 'New Document Alias URL (ID:' +  CAST(nDocID AS NVARCHAR(10)) + ',VER:' + CAST(nVersion AS NVARCHAR(10)) + ') conflicts with Active Document Page URL (ID:' +  CAST(aDocID AS NVARCHAR(10)) + ',VER:' + CAST(aVersion AS NVARCHAR(10)) + ')<br/>'
							WHEN 'AOA' THEN 'New Document Alias URL (ID:' +  CAST(nDocID AS NVARCHAR(10)) + ',VER:' + CAST(nVersion AS NVARCHAR(10)) + ') conflicts with Active Document Alias URL (ID:' +  CAST(aDocID AS NVARCHAR(10)) + ',VER:' + CAST(aVersion AS NVARCHAR(10)) + ')<br/>'
							WHEN 'ARA' THEN 'New Document Alias URL (ID:' +  CAST(nDocID AS NVARCHAR(10)) + ',VER:' + CAST(nVersion AS NVARCHAR(10)) + ') conflicts with Recycled Document Alias URL (ID:' +  CAST(aDocID AS NVARCHAR(10)) + ',VER:' + CAST(aVersion AS NVARCHAR(10)) + ')<br/>'
						END
		  FROM @dupLinks;
		
		RAISERROR ('Duplicate URLs exist.  This must be resolved before the document can be created or activated:<br/><br/>%s', 16, 1, @dups);
	END;
    
    EXEC dbo.internal_ActivatePublicClassifications @docsToOp; 

	/* For the docs we have to activate, we want to check the outgoing active document 
		for a primary LinkParameter that should be added to DocumentAlias. We don't include 
		Linked Pages, because we don't keep aliases for Linked Pages. We don't search
		all old versions because we don't want LPs that have been deliberately removed */
	WITH OldVersionUrls AS (
		SELECT DISTINCT oldOne.DocID, oldOne.LinkParameter
		  FROM @docs docs
		  JOIN Document
		    ON Document.DocID = docs.tDocID
		  JOIN NavigationVersion oldOne
			ON oldOne.DocID = docs.tDocID
		   AND oldOne.Version = Document.ActiveVersion
		  JOIN NavigationVersion newOne
			ON newOne.DocID = docs.tDocID
		   AND newOne.Version = docs.tVersion
		   AND newOne.LinkParameter != oldOne.LinkParameter
		  JOIN DocumentVersion
			ON DocumentVersion.DocID = docs.tDocID
		   AND DocumentVersion.Version = docs.tVersion
		  JOIN DocumentType 
			ON DocumentType.DocumentTypeID = DocumentVersion.DocumentTypeID
		   AND DocumentType.SiteID = Document.SiteID
		 WHERE DocumentType.SupportsUrlRewriting = 1
		   AND (DocumentType.DocumentTypeID = 1 OR DocumentType.DocumentTypeID > 8)
		   AND Document.SiteID != -1
	)
	INSERT INTO DocumentAlias (DocID, LinkParameter)
		SELECT DISTINCT OldVersionUrls.DocID, OldVersionUrls.LinkParameter
		  FROM OldVersionUrls
		 WHERE LinkParameter NOT IN (SELECT LinkParameter
									   FROM DocumentAlias
									  WHERE DocID = OldVersionUrls.DocID);
									  
    UPDATE Document
       SET ActiveVersion = tVersion
      FROM @docs
     WHERE DocID = tDocID;
     
    UPDATE NavigationVersion
       SET GoLiveDate = @actionTime
      FROM @docs
     WHERE DocID = tDocID
       AND Version = tVersion;

    -- Rebuild the XML with just the documents we processed.
    SET @docsToActivate =
        (SELECT tVersion AS '@Version', tDocID AS '*'
           FROM @docs
            FOR XML PATH('Docs'), ROOT('DocID'), TYPE);
    EXEC dbo.internal_WriteWorkstationAuditEvent @eventID, 'InternalFunction', 'internal_Activate', 'Actual Doc Set', @docsToActivate;
END;
GO

IF EXISTS (SELECT 1
			 FROM DocumentAlias
			 JOIN DocumentVersion
			   ON DocumentVersion.DocID = DocumentAlias.DocID
			  AND DocumentVersion.DocumentTypeID > 1 
			  AND DocumentVersion.DocumentTypeID <= 8)
BEGIN
	DELETE 
	  FROM DocumentAlias
	  FROM DocumentVersion
	 WHERE DocumentVersion.DocID = DocumentAlias.DocID
	   AND DocumentVersion.DocumentTypeID > 1 
	   AND DocumentVersion.DocumentTypeID <= 8
END;
GO